<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\aws_cloud\Entity\Ec2\Instance;
use Drupal\cloud\Entity\CloudContentEntityBase;

/**
 * Provides an entities stop confirmation form.
 */
class InstanceStopMultipleForm extends AwsCloudProcessMultipleForm {

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $this->checkCloudOrchestratorIp();
    return $this->formatPlural(count($this->selection),
      'Are you sure you want to stop this @item?',
      'Are you sure you want to stop these @items?', [
        '@item' => $this->entityType->getSingularLabel(),
        '@items' => $this->entityType->getPluralLabel(),
      ]
    );
  }

  /**
   * Check if Cloud Orchestrator is being stopped.
   *
   * Notify the user with a warning message if Cloud Orchestrator's IP
   * is one of the selected IPs.
   */
  private function checkCloudOrchestratorIp(): void {
    foreach ($this->selection as $id => $selection) {
      $instance = Instance::load($id);
      if (!empty($instance) && $this->ec2Service->isInstanceCloudOrchestrator($instance->getInstanceId()) === TRUE) {
        $this->messenger()->addWarning($this->t('Attempting to stop Cloud Orchestrator instance.  Cloud Orchestrator will stop responding until the instance is restarted.'));
        break;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {

    return $this->t('Stop');
  }

  /**
   * {@inheritdoc}
   */
  protected function processCloudResource(CloudContentEntityBase $entity) {

    $this->ec2Service->setCloudContext($entity->getCloudContext());

    return $this->ec2Service->stopInstances([
      'InstanceIds' => [$entity->getInstanceId()],
    ]);
  }

  /**
   * {@inheritdoc}
   */
  protected function processEntity(CloudContentEntityBase $entity) {}

  /**
   * Process an entity and related AWS resource.
   *
   * @param \Drupal\cloud\Entity\CloudContentEntityBase $entity
   *   An entity object.
   *
   * @return bool
   *   Succeeded or failed.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  protected function process(CloudContentEntityBase $entity) {

    if (!empty($entity) && $this->processCloudResource($entity)) {

      $this->processEntity($entity);
      $this->processOperationStatus($entity, 'stopped');

      return TRUE;
    }

    $this->processOperationErrorStatus($entity, 'stopped');

    return FALSE;
  }

  /**
   * Returns the message to show the user after an item was processed.
   *
   * @param int $count
   *   Count of processed translations.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The item processed message.
   */
  protected function getProcessedMessage($count) {
    $this->ec2Service->updateInstances();
    return $this->formatPlural($count, 'Stopped @count item.', 'Stopped @count items.');
  }

}
