<?php

namespace Drupal\aws_cloud\Form\Vpc;

use Drupal\aws_cloud\Form\Ec2\AwsCloudContentForm;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the Carrier Gateway entity create form.
 *
 * @ingroup aws_cloud
 */
class CarrierGatewayCreateForm extends AwsCloudContentForm {

  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    $form = parent::buildForm($form, $form_state);
    $this->ec2Service->setCloudContext($cloud_context);

    $weight = -50;

    $form['carrier_gateway'] = [
      '#type' => 'details',
      '#title' => $this->t('Carrier Gateway'),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['carrier_gateway']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#required'      => TRUE,
    ];

    $vpcs = $this->ec2Service->getVpcs();
    ksort($vpcs);

    $result = $this->ec2Service->describeCarrierGateways();
    foreach ($result['CarrierGateways'] as $gateway) {
      unset($vpcs[$gateway['VpcId']]);
    }

    $form['carrier_gateway']['vpc_id'] = [
      '#type'          => 'select',
      '#title'         => $this->t('VPC CIDR (ID)'),
      '#options'       => $vpcs,
      '#default_value' => $this->entity->getVpcId(),
      '#required'      => TRUE,
    ];

    unset($form['tags']);

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    $form['actions'] = $this->actions($form, $form_state, $cloud_context);

    return $form;
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\aws_cloud\Service\Ec2\Ec2ServiceException
   */
  public function save(array $form, FormStateInterface $form_state) {
    $this->trimTextfields($form, $form_state);

    $cloud_context = $this->routeMatch->getParameter('cloud_context');

    $entity = $this->entity;
    $entity->setCloudContext($cloud_context);

    $result = $this->ec2Service->createCarrierGateway([
      'VpcId' => $entity->getVpcId(),
    ]);

    if (isset($result['CarrierGateway'])
      && ($entity->setCarrierGatewayId($result['CarrierGateway']['CarrierGatewayId']))
      && ($entity->save())
    ) {
      $this->updateNameAndCreatedByTags($entity, $entity->getCarrierGatewayId());

      // Update the carrier gateway.
      $this->ec2Service->updateCarrierGateways([
        'CarrierGatewayIds' => [$entity->getCarrierGatewayId()],
      ], FALSE);

      $this->processOperationStatus($entity, 'created');
      $this->clearCacheValues();

      $form_state->setRedirect(
        'view.aws_cloud_carrier_gateway.list',
        ['cloud_context' => $entity->getCloudContext()]
      );
    }
    else {

      $this->processOperationErrorStatus($entity, 'created');
    }
  }

}
