<?php

namespace Drupal\aws_cloud\Service\Iam;

/**
 * IamService service interacts with the AWS Cloud IAM API.
 */
interface IamServiceInterface {

  /**
   * Set the cloud context.
   *
   * @param string $cloud_context
   *   Cloud context string.
   */
  public function setCloudContext($cloud_context);

  /**
   * Load and return an IamClient.
   *
   * @param array $credentials
   *   The array of credentials.
   *
   * @return \Aws\Iam\IamClient|null
   *   If the $params is empty or $iam_client (Iam2Client) is NULL.
   */
  public function getIamClient(array $credentials = []);

  /**
   * Get instance profiles.
   *
   * @param array $params
   *   Array of parameters.
   * @param array $credentials
   *   The array of credentials.
   * @param bool $show_errors
   *   Show IAM errors.
   *
   * @return array
   *   Instance profiles.
   */
  public function listInstanceProfiles(array $params = [], array $credentials = [], $show_errors = TRUE);

}
