<?php

namespace Drupal\aws_cloud\Traits;

/**
 * The trait for AWS Cloud Form.
 */
trait AwsCloudFormTrait {

  /**
   * Max length of the description of AWS Cloud security group (rule).
   *
   * @var int
   */
  public static $awsCloudSecurityGroupDescriptionMaxLength = 255;

  /**
   * Special characters in the description of AWS Cloud security group (rule).
   *
   * @var string
   */
  public static $awsCloudSecurityGroupDescriptionSpecialCharacters = '. _-:/()#,@[]+=&;{}!$*';

  /**
   * Get the pattern of a security group (rule) description.
   *
   * @return string
   *   A regexp pattern excluding ^ and $.
   */
  protected function getPatternOfSecurityGroupDescription(): string {
    return '[A-Za-z0-9' . preg_quote(self::$awsCloudSecurityGroupDescriptionSpecialCharacters, NULL) . ']*';
  }

  /**
   * Get the AWS Cloud service providers as an array.
   *
   * @return array
   *   An array of AWS Cloud service providers.
   */
  protected function getCloudServiceProviderOptions(): array {
    $entities = \Drupal::service('plugin.manager.cloud_config_plugin')->loadConfigEntities('aws_cloud');

    $cloud_configs = [];
    foreach ($entities ?: [] as $entity) {
      $cloud_configs[] = $entity->getCloudContext();
    }

    return $cloud_configs;
  }

  /**
   * Get all other criteria options on AWS Cloud Notifications.
   *
   * @return array
   *   Array of criteria.
   */
  protected function getCriteriaOptions(): array {
    return [30, 60, 90, 180, 365];
  }

  /**
   * Get the frequency options on AWS Cloud Notifications.
   *
   * @return array
   *   Array of frequency.
   */
  protected function getFrequencyOptions(): array {
    return [86400, 604800, 2592000, 5184000, 7776000, 15552000, 31104000];
  }

  /**
   * Get the items per page on Pager options, Views, AWS Cloud Settings.
   *
   * @return array
   *   Array of item per page.
   */
  protected function getItemsPerPageOptions(): array {
    return [10, 15, 20, 25, 50, 100];
  }

  /**
   * Get the notification criteria options on AWS Cloud Notifications.
   *
   * @return array
   *   Array of notification criteria.
   */
  protected function getNotificationCriteriaOptions(): array {
    return [1, 30, 60, 90, 180, 365];
  }

  /**
   * Get all period options on AWS Cloud Notifications.
   *
   * @return array
   *   Array of period.
   */
  protected function getPeriodOptions(): array {
    return ['4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14'];
  }

}
