(function ($) {
  'use strict';

  if (!drupalSettings.k8s || !drupalSettings.k8s.k8s_node_allocated_resources_chart_json_url) {
    return;
  }

  let chart_period = drupalSettings.k8s.k8s_node_allocated_resources_chart_period || 1;

  $('#k8s_node_allocated_resources_chart').append('<div id="node_allocated_resources_chart_options">');
  $('#node_allocated_resources_chart_options').css('font-size', 'x-small');
  $('#node_allocated_resources_chart_options').css('text-align', 'right');
  $('#node_allocated_resources_chart_options').append('<span>');
  $('#node_allocated_resources_chart_options').append('<span>');

  if (drupalSettings.k8s.k8s_node_allocated_resources_chart_periods_json_url) {
    let chart_periods_json_url = drupalSettings.k8s.k8s_node_allocated_resources_chart_periods_json_url;
    $.get(chart_periods_json_url, function (json) {
      $('#node_allocated_resources_chart_options > span:last-child').append('<label for="node_allocated_resources_chart_period">' + Drupal.t('Chart Period') + '</label>');
      $('#node_allocated_resources_chart_options > span:last-child label').css('margin-left', '10px');
      $('#node_allocated_resources_chart_options > span:last-child').append('<select id="node_allocated_resources_chart_period">');
      $.each(json, function (index, value) {
        $('#node_allocated_resources_chart_period').append('<option value="' + index + '">' + value + '</option>');
      });
      $('#node_allocated_resources_chart_period option').css('font-size', 'x-small');

      if (node_allocated_resources_chart_period && $('select#node_allocated_resources_chart_period option[value="' + chart_period + '"]').length > 0) {
        $('#node_allocated_resources_chart_period').val(chart_period);
      }
      else if ($('select#node_allocated_resources_chart_period option[value="1"]').length > 0) {
        $('#node_allocated_resources_chart_period').val(1);
      }
      $('#node_allocated_resources_chart_period').change(function () {
        updateChart();
      });
    });
  }

  let createData = function (json) {
    let resource_data = [];
    resource_data['cpu'] = [];
    resource_data['memory'] = [];
    resource_data['pod'] = [];
    json.forEach(function (data) {
      let cpu_capacity = data.cpu.cpu_capacity;
      let cpu_usage = data.cpu.cpu_usage;
      let cpu_usage_ratio = cpu_capacity > 0 ? cpu_usage / cpu_capacity : 0;
      let cpu_usage_rounded = cpu_usage.toFixed(2);
      resource_data['cpu'].push({
        t: data.cpu.timestamp * 1000,
        y: (cpu_usage_ratio * 100).toFixed(1),
        usage: cpu_usage_rounded + ' / ' + cpu_capacity + ' Cores',
      });
      let memory_capacity = data.memory.memory_capacity;
      let memory_usage = data.memory.memory_usage;
      let memory_usage_rounded = (memory_usage / 1024 / 1024 / 1024).toFixed(2);
      let memory_capacity_rounded = (memory_capacity / 1024 / 1024 / 1024).toFixed(2);
      let memory_usage_ratio = memory_capacity > 0 ? memory_usage / memory_capacity : 0;
      resource_data['memory'].push({
        t: data.memory.timestamp * 1000,
        y: (memory_usage_ratio * 100).toFixed(1),
        usage: memory_usage_rounded + ' / ' + memory_capacity_rounded + ' GiB',
      });
      let pod_capacity = data.pod.pod_capacity;
      let pod_usage = data.pod.pod_usage;
      let pod_usage_ratio = pod_capacity > 0 ? pod_usage / pod_capacity : 0;
      resource_data['pod'].push({
        t: data.cpu.timestamp * 1000,
        y: (pod_usage_ratio * 100).toFixed(1),
        usage: pod_usage + ' / ' + pod_capacity + ' Pods',
      });
    });
    return resource_data;
  }

  let json_url = drupalSettings.k8s.k8s_node_allocated_resources_chart_json_url;

  let initChart = function () {
    let init_json_url = json_url + '?period=' + chart_period;
    d3.json(init_json_url).then(function (json) {
      resource_chart.data.datasets = [];
      let data = createData(json);
      let color = Chart.helpers.color;
      resource_chart.data.datasets.push({
        label: 'CPU Core Usage',
        backgroundColor: color('green').alpha(0.5).rgbString(),
        borderColor: 'green',
        data: data['cpu'],
        type: 'line',
        pointRadius: 0,
        fill: false,
        lineTension: 0,
        borderWidth: 2
      });
      resource_chart.data.datasets.push({
        label: 'Memory Usage',
        backgroundColor: color('blue').alpha(0.5).rgbString(),
        borderColor: 'blue',
        data: data['memory'],
        type: 'line',
        pointRadius: 0,
        fill: false,
        lineTension: 0,
        borderWidth: 2
      });
      resource_chart.data.datasets.push({
        label: 'Pods Allocation',
        backgroundColor: color('red').alpha(0.5).rgbString(),
        borderColor: 'red',
        data: data['pod'],
        type: 'line',
        pointRadius: 0,
        fill: false,
        lineTension: 0,
        borderWidth: 2
      });
      resource_chart.update();
    });
  };

  let updateChart = function () {
    let update_json_url = json_url + '?period=' + $('#node_allocated_resources_chart_period').val();
    d3.json(update_json_url).then(function (json) {
      let data = createData(json);
      resource_chart.data.datasets[0].data = data['cpu'];
      resource_chart.data.datasets[1].data = data['memory'];
      resource_chart.data.datasets[2].data = data['pod'];
      resource_chart.update();
    });
  };

  // Build chart.
  $('#k8s_node_allocated_resources_chart').append('<div class="col-sm-9" style="float: none; margin: 0 auto"><canvas id="resource_chart"></canvas></div>');
  let font_color = $('.panel-title').css("color");
  let resource_ctx = document.getElementById('resource_chart').getContext('2d');
  let resource_cfg = {
    type: 'bar',
    options: {
      legend: {
        labels: {
          fontColor: font_color,
        }
      },
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'day',
            displayFormats: {
              hour: 'MMM D hA'
            }
          },
          distribution: 'linear',
          bounds: 'ticks',
          ticks: {
            source: 'ticks',
            autoSkip: false,
            fontColor: font_color,
          }
        }],
        yAxes: [{
          scaleLabel: {
            display: true,
            labelString: 'Resource Usage (%)',
            fontColor: font_color,
          },
          ticks: {
            source: 'ticks',
            autoSkip: false,
            fontColor: font_color,
          }
        }]
      },
      tooltips: {
        intersect: false,
        mode: 'nearest',
        callbacks: {
          label: function (tooltipItem, myData) {
            let label = myData.datasets[tooltipItem.datasetIndex].label || '';
            if (label) {
              label += ': ';
            }
            let usage = myData.datasets[tooltipItem.datasetIndex].data[tooltipItem.index].usage;
            label += usage + ' (';
            label += parseFloat(tooltipItem.value).toFixed(1);
            label += '%)';
            return label;
          }
        }
      }
    }
  };

  let resource_chart = new Chart(resource_ctx, resource_cfg);
  console.log(resource_chart);

  initChart();

  let interval = drupalSettings.k8s.k8s_js_refresh_interval || 10;
  setInterval(function () {
    updateChart();
  }, interval * 1000);

})(jQuery);
