<?php

namespace Drupal\openstack\Service;

use Drupal\cloud\Entity\CloudConfigInterface;

/**
 * Provides OpenStackServiceInterface interface.
 */
interface OpenStackServiceInterface {

  public const IPV4 = 4;

  /**
   * Get Client object for API execution.
   *
   * @param array $credentials
   *   The array of credentials.
   *
   * @return array
   *   The client.
   */
  public function getClient(array $credentials = []);

  /**
   * Set the cloud context.
   *
   * @param string $cloud_context
   *   Cloud context string.
   */
  public function setCloudContext($cloud_context);

  /**
   * Get Instance List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Instance list API response.
   */
  public function describeInstances(array $params = []);

  /**
   * Update Instance.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Instance update API response.
   */
  public function modifyLaunchTemplate(array $params = []);

  /**
   * Start Instance.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   The Instance Start API response.
   */
  public function startInstances(array $params = []);

  /**
   * Stop Instance.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   The Instance Stop API response.
   */
  public function stopInstances(array $params = []);

  /**
   * Reboot Instance.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   The Instance Reboot API response.
   */
  public function rebootInstances(array $params = []);

  /**
   * Terminate|Delete Instance.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   The Instance Terminate|Delete API response.
   */
  public function terminateInstance(array $params = []);

  /**
   * Run instance.
   *
   * @param array $params
   *   Parameters to pass to the API.
   * @param array $tags
   *   The tags array.
   *
   * @return bool
   *   The create instance list API response.
   */
  public function runInstances(array $params = [], array $tags = []);

  /**
   * Get Image List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Image list API response.
   */
  public function describeImages(array $params = []);

  /**
   * Create Image List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Image create API response.
   */
  public function createImage(array $params = []);

  /**
   * Delete Image List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Image delete API response.
   */
  public function deregisterImage(array $params = []);

  /**
   * Get key pairs List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The key pairs list API response.
   */
  public function describeKeyPairs(array $params = []);

  /**
   * Create key pairs List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The key pairs create API response.
   */
  public function createKeyPair(array $params = []);

  /**
   * Import key pairs List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The key pairs import API response.
   */
  public function importKeyPair(array $params = []);

  /**
   * Delete key pairs List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   The key pairs delete API response.
   */
  public function deleteKeyPair(array $params = []);

  /**
   * Get volume list.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The volume list API response.
   */
  public function describeVolumes(array $params = []);

  /**
   * Create volume.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The volume create API response.
   */
  public function createVolume(array $params = []);

  /**
   * Delete volume.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   Succeeded or failed.
   */
  public function deleteVolume(array $params = []);

  /**
   * Attach volume.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   Succeeded or failed.
   */
  public function attachVolume(array $params = []);

  /**
   * Detach volume.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   Succeeded or failed.
   */
  public function detachVolume(array $params = []);

  /**
   * Get snapshot list.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The snapshot list API response.
   */
  public function describeSnapshots(array $params = []);

  /**
   * Create snapshot.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The snapshot create API response.
   */
  public function createSnapshot(array $params = []);

  /**
   * Delete snapshot.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   Succeeded or failed.
   */
  public function deleteSnapshot(array $params = []);

  /**
   * Get security group list.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The security group list API response.
   */
  public function describeSecurityGroups(array $params = []);

  /**
   * Create security group list.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The security group create API response.
   */
  public function createSecurityGroup(array $params = []);

  /**
   * Delete security group List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   Succeeded or failed.
   */
  public function deleteSecurityGroup(array $params = []);

  /**
   * Get network interface list.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The network interface list API response.
   */
  public function describeNetworkInterfaces(array $params = []);

  /**
   * Create network interface.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The network interface create API response.
   */
  public function createNetworkInterface(array $params = []);

  /**
   * Delete network interface.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   Succeeded or failed.
   */
  public function deleteNetworkInterface(array $params = []);

  /**
   * Get Floating IP List.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Floating IP list API response.
   */
  public function describeAddresses(array $params = []);

  /**
   * Create Floating IP.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Floating IP create API response.
   */
  public function allocateAddress(array $params = []);

  /**
   * Associate Floating IP.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Floating IP associate API response.
   */
  public function associateAddress(array $params = []);

  /**
   * Disassociate Floating IP.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return array
   *   The Floating IP associate API response.
   */
  public function disassociateAddress(array $params = []);

  /**
   * Delete Floating IP.
   *
   * @param array $params
   *   Parameters to pass to the API.
   *
   * @return bool
   *   The Floating IP delete API response.
   */
  public function releaseAddress(array $params = []);

  /**
   * Call the API for updated entities and store them as Instance entities.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateInstanceEntities(array $params = [], $clear = TRUE, $entity_type = '', $cloud_context = ''): bool;

  /**
   * Update the Instances.
   *
   * @param array $params
   *   Parameters array to send to API.
   * @param bool $clear
   *   TRUE to clear stale instances.
   *
   * @return bool
   *   Indicates success or failure.
   */
  public function updateInstances(array $params = [], $clear = TRUE): bool;

  /**
   * Update the OpenStackImages.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateImages(array $params = [], $clear = TRUE): bool;

  /**
   * Call the API for updated entities and store them as Image entities.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateImageEntities(array $params = [], $clear = TRUE, $entity_type = '', $cloud_context = ''): bool;

  /**
   * Call the API for updated entities and store them as key pair entities.
   *
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   * @param array $params
   *   Key Name info.
   * @param bool $update
   *   TRUE to update Key info.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateKeyPairEntities($entity_type = '', $cloud_context = '', array $params = [], $update = TRUE): bool;

  /**
   * Update the OpenStack key pairs.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateKeyPairs(): bool;

  /**
   * Call the API for updated entities and store them as volume entities.
   *
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   * @param array $params
   *   Volume info.
   * @param bool $update
   *   TRUE to update volume.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateVolumeEntities($entity_type = '', $cloud_context = '', array $params = [], $update = TRUE): bool;

  /**
   * Update the volumes.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateVolumes(): bool;

  /**
   * Get snapshot id and name.
   *
   * @param array $volumes
   *   The volume array.
   *
   * @return array
   *   Return map array.
   */
  public function getSnapshotIdNameMap(array $volumes = []);

  /**
   * Call the API for updated entities and store them as snapshot entities.
   *
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   * @param array $params
   *   Snapshot IDs.
   * @param bool $update
   *   TRUE to update info.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateSnapshotEntities($entity_type = '', $cloud_context = '', array $params = [], $update = TRUE): bool;

  /**
   * Update the snapshots.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateSnapshots(): bool;

  /**
   * Update the OpenStackSecurityGroups.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateSecurityGroups(array $params = [], $clear = TRUE): bool;

  /**
   * Call the API for updated entities and store them as SecurityGroup entities.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   * @param bool $update
   *   TRUE to update SG info.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateSecurityGroupEntities(array $params = [], $clear = TRUE, $entity_type = '', $cloud_context = '', $update = TRUE): bool;

  /**
   * Update the OpenStackNetworkInterfaces.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateNetworkInterfaces(array $params = [], $clear = TRUE): bool;

  /**
   * Call API for updated entities and store them as NetworkInterface entities.
   *
   * @param array $params
   *   Optional parameters array.
   * @param bool $clear
   *   TRUE to clear stale security groups.
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   * @param bool $update
   *   TRUE to update info.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateNetworkInterfaceEntities(array $params = [], $clear = TRUE, $entity_type = '', $cloud_context = '', $update = TRUE): bool;

  /**
   * Update the OpenStackFloatingIps.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateFloatingIps(): bool;

  /**
   * Call the API for updated entities and store them as FloatingIp entities.
   *
   * @param string $entity_type
   *   Entity type string.
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return bool
   *   Indicates success so failure.
   */
  public function updateFloatingIpEntities($entity_type = '', $cloud_context = ''): bool;

  /**
   * Setup IP Permissions.
   *
   * @param object $security_group
   *   The security group entity.
   * @param string $field
   *   Field to used for lookup.
   * @param array $ec2_permissions
   *   Permissions array from Ec2.
   */
  public function setupIpPermissions(&$security_group, $field, array $ec2_permissions);

  /**
   * Setup the ip_permission field given the inbound security group array.
   *
   * The array comes from DescribeSecurityGroup Amazon EC2 API call.
   *
   * @param array $ec2_permission
   *   An array object of EC2permission.
   *
   * @return array
   *   An array of \Drupal\Core\Field\FieldItemInterface.
   */
  public function setupIpPermissionObject(array $ec2_permission);

  /**
   * Get Tag name.
   *
   * @param array $openstack_obj
   *   The openstack object.
   * @param string $default_value
   *   The default value.
   *
   * @return string
   *   Return name of corresponding resource.
   */
  public function getTagName(array $openstack_obj, $default_value);

  /**
   * Get UID tag value.
   *
   * @param array $tags_array
   *   The tags array of API.
   * @param string $entity_type_id
   *   The entity type id.
   *
   * @return int
   *   Return user id.
   */
  public function getUidTagValue(array $tags_array, $entity_type_id);

  /**
   * Get instance uid.
   *
   * @param string $instance_id
   *   The instance id.
   *
   * @return int
   *   Return user id.
   */
  public function getInstanceUid($instance_id);

  /**
   * Method to clear all entities out of the system.
   */
  public function clearAllEntities();

  /**
   * Create queue items for update resources queue.
   *
   * @param \Drupal\cloud\Entity\CloudConfigInterface $cloud_config
   *   The Cloud Config entity.
   */
  public function createResourceQueueItems(CloudConfigInterface $cloud_config): void;

}
