<?php

namespace Drupal\vmware\Form;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\cloud\Entity\CloudContentEntityBase;

/**
 * Provides an entities suspend confirmation form.
 */
class VmwareVmSuspendMultipleForm extends VmwareProcessMultipleForm {

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {

    return $this->formatPlural(count($this->selection),
      'Are you sure you want to suspend this @item?',
      'Are you sure you want to suspend these @items?', [
        '@item' => $this->entityType->getSingularLabel(),
        '@items' => $this->entityType->getPluralLabel(),
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText(): TranslatableMarkup {
    return $this->t('Suspend');
  }

  /**
   * {@inheritdoc}
   */
  protected function processCloudResource(CloudContentEntityBase $entity): bool {
    $this->vmwareService->setCloudContext($entity->getCloudContext());
    $this->vmwareService->login();
    $this->vmwareService->suspendVm([
      'VmId' => $entity->getVmId(),
    ]);
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  protected function processEntity(CloudContentEntityBase $entity): void {}

  /**
   * Process an entity and related VMware resource.
   *
   * @param \Drupal\cloud\Entity\CloudContentEntityBase $entity
   *   An entity object.
   *
   * @return bool
   *   Succeeded or failed.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  protected function process(CloudContentEntityBase $entity): bool {
    if (empty($entity)) {
      return FALSE;
    }

    try {
      $this->processCloudResource($entity);
      $this->processEntity($entity);
      $this->processOperationStatus($entity, 'suspended');

      return TRUE;
    }
    catch (\Exception $e) {
      $this->processOperationErrorStatus($entity, 'suspended');
      return FALSE;
    }
  }

  /**
   * Returns the message to show the user after an item was processed.
   *
   * @param int $count
   *   Count of processed translations.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The item processed message.
   */
  protected function getProcessedMessage($count): TranslatableMarkup {
    $this->vmwareService->updateVms();
    return $this->formatPlural($count, 'Suspended @count item.', 'Suspendped @count items.');
  }

}
