<?php

namespace Drupal\cloud\Entity;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides dynamic permissions for Cloud.
 */
class CloudPermissions implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs new CloudPermissions object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ModuleHandlerInterface $moduleHandler) {
    $this->entityTypeManager = $entityTypeManager;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('module_handler')
    );
  }

  /**
   * Returns an array of cloud service provider (CloudConfig) permissions.
   */
  public function configPermissions() {
    $permissions = [];
    $cloud_configs = $this->entityTypeManager->getStorage('cloud_config')->loadMultiple();
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $label = $cloud_config->label();
      $permissions['view ' . $cloud_config->getCloudContext()] = [
        'title' => $this->t('Access %label', ['%label' => $label]),
        'description' => $this->t('Allows access to entities in %label. Entity permissions such as e.g. <em>List %module_name &lt;entity name&gt;</em> and <em>View own %module_name &lt;entity name&gt;</em> (or <em>View any %module_name &lt;entity name&gt;</em>) need to be granted in conjunction. Otherwise, no entities will be shown to the user.', [
          '%module_name' => $this->moduleHandler->getName($cloud_config->bundle()),
          '%label' => $label,
        ]),
      ];
    }
    return $permissions;
  }

}
