<?php

namespace Drupal\Tests\cloud\Traits;

/**
 * The assertion trait for common usage.
 */
trait CloudAssertionTrait {

  /**
   * Assert successful/non-error responses.
   */
  protected function assertNoErrorMessage(): void {
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains($this->t('Error message'));
    $this->assertSession()->pageTextNotContains($this->t('Warning message'));
  }

  /**
   * Assert warning responses.
   */
  protected function assertWarningMessage(): void {
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains($this->t('Error message'));
    $this->assertSession()->pageTextContains($this->t('Warning message'));
  }

  /**
   * Assert error response.
   */
  protected function assertErrorMessage(): void {
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains($this->t('Warning message'));
    $this->assertSession()->pageTextContains($this->t('Error message'));
  }

  /**
   * Assert error response.
   */
  protected function assertWarningAndErrorMessage(): void {
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains($this->t('Warning message'));
    $this->assertSession()->pageTextContains($this->t('Error message'));
  }

  /**
   * Assert error message only with invalid Google Apps.
   */
  protected function assertGoogleAppErrorOnly(): void {
    $errors = $this->xpath('//div[contains(@class,"messages--error")]/div/div[@class="messages__content"]');
    $expected = $this->t('The module Google Applications is invalid. Please enable the module.');
    $this->assertEquals($errors[0]->getText(), $expected);
  }

}
